/*
 * Copyright 2020 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.devsite.components.impl

import com.google.devsite.components.render
import com.google.devsite.components.symbols.ClasslikeDescription
import kotlinx.html.FlowContent
import kotlinx.html.hr
import kotlinx.html.p
import kotlinx.html.pre

internal data class DefaultClasslikeDescription(
    override val data: ClasslikeDescription.Params,
) : ClasslikeDescription {

    override fun render(into: FlowContent) = into.run {
        data.header?.render(this)
        p {
            pre {
                data.primarySignature.render(this)
            }
        }
        data.hierarchy.render(this)
        data.relatedSymbols.render(this)
        data.descriptionDocs.render(into, separator = null, header = { hr() })
    }
}
